import streamlit as st
import os
import sys
import signal
import pydeck as pdk
import streamlit_echarts
import streamlit_authenticator as stauth
import yaml
from yaml.loader import SafeLoader
from utils import logger, cached_data, constants, graph_builder
from st_pages import Page, show_pages

st.set_page_config(layout="wide", page_title="Análise dos Dados")
with open("./config.yaml") as file:
    config = yaml.load(file, Loader=SafeLoader)

authenticator = stauth.Authenticate(
    config["credentials"],
    config["cookie"]["name"],
    config["cookie"]["key"],
    config["cookie"]["expiry_days"],
)
pages = [
    Page("app.py", "Dashboard", ":chart_with_upwards_trend:"),
    Page("pages/prediction.py", "Predições", ":crystal_ball:"),
    Page("pages/comparation.py", "Comparação", ":vs:"),
]
authenticator.login("Login", "main")
if st.session_state["authentication_status"]:
    show_pages(pages)
    st.sidebar.write(f'Olá **{st.session_state["name"]}** 👑!')
    authenticator.logout("Logout", "sidebar")
    logger_utils = logger.Logger(st.session_state["name"])

    parquet_file = ""
    try:
        parquet_file = sys.argv[1]
    except IndexError:
        logger_utils.log("Usage: streamlit run app.py <parquet_file> [...args]")
        os.kill(os.getpid(), signal.SIGKILL)

    cached_data_utils = cached_data.CacheData(st.session_state["name"], parquet_file)

    parquet_file = cached_data_utils.get_parquet_file()
    constants_utils = constants.Constants()
    graph_builder_utils = graph_builder.GraphBuilder()
    logger_utils.log(
        "############################### APP STARTED ################################"
    )
    df = cached_data_utils.load_df()

    st.sidebar.title("Movimento de pessoas no Aeroporto de Lisboa")
    years_options = cached_data_utils.get_available_years(df)
    st.sidebar.selectbox(
        "Ano", years_options, key="year", disabled=len(years_options) == 1
    )
    months_options = cached_data_utils.get_available_months(df, st.session_state.year)
    st.sidebar.multiselect(
        "Mês",
        months_options,
        default=months_options[:3],
        max_selections=6,
        key="months",
    )
    time_selection = st.sidebar.slider(
        "Horas", min_value=0, max_value=24, value=(0, 24)
    )
    c1_and_or_c2_option = st.sidebar.selectbox(
        "Colunas",
        list(constants_utils.c_columns.values()),
        key="turists_option",
        index=2,
    )

    logger_utils.log(f"Selected year: {st.session_state.year}")
    logger_utils.log(f"Selected months: {st.session_state.months}")
    logger_utils.log(f"Selected min time: {time_selection[0]}")
    logger_utils.log(f"Selected max_time: {time_selection[1]}")
    logger_utils.log(f"Selected c1_and_or_c2: {c1_and_or_c2_option}")

    search_button = st.sidebar.button(
        "Procurar", disabled=(len(st.session_state.months) == 0)
    )

    if search_button:
        with st.spinner("A carregar os dados..."):
            df_filtered = cached_data_utils.filter_df(
                df,
                st.session_state.year,
                st.session_state.months,
                time_selection[0],
                time_selection[1],
                st.session_state.turists_option,
            )
            logger_utils.log(
                f"Filtered dataframe contains: {len(df_filtered.index)} rows"
            )
        st.title("Relatório da pesquisa")
        # Started building distribution graphs
        logger_utils.log("Started building distribution graphs")
        distribution1, distribution2 = st.columns(2)
        with st.spinner("A construir os gráficos de distribuição"):
            distribution_graph_utils = graph_builder_utils.get_distribution_df(
                df_filtered, st.session_state["turists_option"]
            )
            distribution_df = distribution_graph_utils[0]
            distribution_initial_values = distribution_graph_utils[1]
            with distribution1:
                grid_layer = pdk.Layer(
                    "GridLayer",
                    distribution_df,
                    pickable=True,
                    extruded=True,
                    cell_size=200,
                    get_position=["Longitude", "Latitude"],
                )
                grid_deck = pdk.Deck(
                    initial_view_state=distribution_initial_values,
                    layers=[grid_layer],
                    # If you're seeing an error in this line it's normal. Python intellisense is not recognizing the dict a possible parameter to tooltip
                    # But, as you can see in the library documentation: "tooltip : bool or dict of {str: str}, default True"
                    tooltip={
                        "html": "<b>Localização</b>:{position}<br><b>Total<b>: {count}"
                    },
                )
                st.subheader("Distribuição com grid layer")
                st.pydeck_chart(grid_deck)
            with distribution2:
                heat_layer = pdk.Layer(
                    "HeatmapLayer",
                    distribution_df,
                    get_position=["Longitude", "Latitude"],
                )
                heat_deck = pdk.Deck(
                    initial_view_state=distribution_initial_values, layers=[heat_layer]
                )
                st.subheader("Distribuição com heatmap")
                st.pydeck_chart(heat_deck)
        # Finished building the distribution graphs
        ##################################
        ##################################
        months_labels = list(df_filtered["Month"].unique())
        sorted_months_labels = {}
        for month in months_labels:
            sorted_months_labels[
                list(constants_utils.static_months_map.keys())[
                    list(constants_utils.static_months_map.values()).index(month)
                ]
            ] = month
        months_labels = dict(sorted(sorted_months_labels.items())).values()
        # Start building C1 graph
        if "Nacionais" in st.session_state.turists_option:
            with st.spinner("A carregar os dados dos turistas :red[sem] roaming"):
                non_roamers_graph = graph_builder_utils.build_total_people_chart(
                    df_filtered, months_labels, "Nacionais"
                )
                st.subheader("Dados dos turistas nacionais")
                st.plotly_chart(non_roamers_graph)
        # Finished building C1 graph
        ##################################
        ##################################
        # Start building C2 graph
        if "Roamers" in st.session_state.turists_option:
            with st.spinner("A carregar os dados dos turistas :red[com] roaming"):
                total_roamers_graph = graph_builder_utils.build_total_people_chart(
                    df_filtered, months_labels, "Roamers"
                )
                st.subheader("Dados dos turistas roamers")
                st.plotly_chart(total_roamers_graph)
        # Finished building C2 graph
        ##################################
        ##################################
        # Start building E6 graph
        with st.spinner("A carregar o top de aplicações"):
            logger_utils.log("Searching top 10 applications")
            top_applications_chart = graph_builder_utils.build_top_applications_chart(
                df_filtered
            )
            st.subheader("Top 10 aplicações mais usadas (Utilizadores de Android)")
            st.plotly_chart(top_applications_chart)
        # Finished building E6 graph
        ##################################
        ##################################
        # Start building D1 graph
        if "Roamers" in st.session_state.turists_option:
            with st.spinner("A carregar o top de nacionalidades conhecidas"):
                logger_utils.log("Searching top 10 nacionalities")
                nationalities_graph = graph_builder_utils.build_top_nationalities_chart(
                    df_filtered, months_labels
                )
                st.subheader("Top 10 nacionalidade conhecidas")
                streamlit_echarts.st_echarts(nationalities_graph)
        # Finished building D1 graph
        ##################################
        ##################################
        # Started building evolution graph
        with st.spinner("A avaliar a evolução ao longo do tempo"):
            logger_utils.log("Building evolution graphs")
            evolution_graph = graph_builder_utils.build_total_people_evolution_chart(
                df_filtered, time_selection[0], time_selection[1]
            )
            st.subheader("Mediana do total de turistas ao longo do tempo")
            streamlit_echarts.st_echarts(evolution_graph)
        # Finished building evolution graph
        ##################################
        ##################################
elif st.session_state["authentication_status"] is False:
    st.error("Username/password incorretos")
else:
    show_pages([pages[0]])