import streamlit as st
import pandas as pd
import os
import signal
from datetime import datetime
from utils import logger, constants
from pathlib import Path

class CacheData:
    def __init__(self, current_user, parquet_file) -> None:
        self.constants = constants.Constants()
        self.base_path = os.path.dirname(Path(__file__).parent)
        self.logger_utils = logger.Logger(current_user)
        self.parquet_file = parquet_file

    @st.cache_data
    def get_parquet_file(_self):
        parquet_file = ""
        try:
            parquet_file = _self.parquet_file
        except IndexError:
            _self.logger_utils.log("Usage: streamlit run app.py <parquet_file> [...args]")
            os.kill(os.getpid(), signal.SIGKILL)

        _self.logger_utils.log(f"Using parquet file: {parquet_file}")

        if not os.path.exists(parquet_file):
            _self.logger_utils.log(f"{parquet_file} not found")
            os.kill(os.getpid(), signal.SIGKILL)

        return parquet_file
    
    @st.cache_data
    def load_df(_self):
        _self.logger_utils.log("Loading dataframe")
        before_load = datetime.now()
        df = pd.read_parquet(_self.base_path + "/" + _self.parquet_file)
        _self.logger_utils.log(
            f"Parquet took {(datetime.now() - before_load).total_seconds()} seconds to be loaded"
        )
        return df

    @st.cache_data
    def filter_df(
        _self,
        dataframe,
        selected_year,
        selected_months,
        selected_min_hour,
        selected_max_hour,
        turists_option,
    ):
        _self.logger_utils.log(
            f"""Filtering dataframe:
            selected_year = {selected_year}, 
            selected_months = {selected_months},
            min_hour = {selected_min_hour},
            max_hour = {selected_max_hour},
            turists_columns = {turists_option}
            """
        )
        selected_months_numbers = []
        for month in selected_months:
            selected_months_numbers.append(
                list(_self.constants.static_months_map.keys())[
                    list(_self.constants.static_months_map.values()).index(month)
                ]
            )
        df_filtered = dataframe[dataframe["Year"] == selected_year]
        df_filtered = df_filtered[df_filtered["Month"].isin(selected_months_numbers)]
        df_filtered = df_filtered[
            (df_filtered["Hour"] >= selected_min_hour)
            & (df_filtered["Hour"] <= selected_max_hour)
        ]
        df_filtered["Month"] = df_filtered["Month"].replace(_self.constants.static_months_map)
        return df_filtered
    
    @st.cache_data
    def get_available_years(_self, dataframe):
        _self.logger_utils.log("Getting available years")
        years = list(dataframe["Year"].unique())
        years.sort()
        _self.logger_utils.log(f"Available years: {years}")
        return years

    @st.cache_data
    def get_available_months(_self, dataframe, selected_year):
        _self.logger_utils.log("Getting available months...")
        df_year_months = list(dataframe[dataframe["Year"] == selected_year]["Month"].unique())
        months = {}
        for m in df_year_months:
            months[m] = _self.constants.static_months_map[m]

        sorted_months = dict(sorted(months.items()))
        available_months = list(sorted_months.values())
        _self.logger_utils.log(f"Available months: {available_months}")
        return available_months