import pandas as pd
import pydeck as pdk
import plotly.express as px
from utils import constants
import numpy

class GraphBuilder:
    def __init__(self) -> None:
        self.constants = constants.Constants()

    def get_distribution_df(self, df, turists_option):
        median_of_each_grid = {}
        grid_coordinates = {}
        grids = list(df["Grid_ID"].unique())
        for grid in grids:
            grid_rows = df[df["Grid_ID"] == grid]
            median_of_each_grid[grid] = {}
            c_option = self.constants.get_c_column_name_by_value(turists_option)
            if c_option == "C1":
                median_of_each_grid[grid]["C1"] = grid_rows["C1"].median()
                median_of_each_grid[grid]["C2"] = 0
            elif c_option == "C2":
                median_of_each_grid[grid]["C1"] = 0
                median_of_each_grid[grid]["C2"] = grid_rows["C2"].median()
            else:
                median_of_each_grid[grid]["C1"] = grid_rows["C1"].median()
                median_of_each_grid[grid]["C2"] = grid_rows["C2"].median()

            grid_coordinates[grid] = {
                "Latitude": grid_rows["Latitude"].values[0],
                "Longitude": grid_rows["Longitude"].values[0],
            }

        distribution_df_dict = {"Grid_ID": [], "Latitude": [], "Longitude": []}
        for grid in grid_coordinates:
            for _ in range(
                int(median_of_each_grid[grid]["C1"] + median_of_each_grid[grid]["C2"])
            ):
                distribution_df_dict["Grid_ID"].append(grid)
                distribution_df_dict["Latitude"].append(
                    grid_coordinates[grid]["Latitude"]
                )
                distribution_df_dict["Longitude"].append(
                    grid_coordinates[grid]["Longitude"]
                )

        distribution_initial_values = pdk.ViewState(
            latitude=distribution_df_dict["Latitude"][0],
            longitude=distribution_df_dict["Longitude"][0],
            zoom=12,
            pitch=35,
        )
        return pd.DataFrame(distribution_df_dict), distribution_initial_values

    def build_total_people_chart(self, df, months_labels, c_column):
        c_column = self.constants.get_c_column_name_by_value(c_column)
        c_total_data = {}
        for month in months_labels:
            month_df = df[df["Month"] == month]
            c_data = []
            for day in list(month_df["Day"].unique()):
                c_data.append(month_df[month_df["Day"] == day][c_column].sum())
            c_total_data[month] = numpy.median(c_data)
        c_total_df = {
            "Month": list(c_total_data.keys()),
            c_column: list(c_total_data.values()),
        }
        return px.bar(
            pd.DataFrame(data=c_total_df),
            x="Month",
            y=c_column,
            labels={"Month": "Mês", c_column: self.constants.c_columns[c_column] + " - Mediana do nº de pessoas/dia"}, # type: ignore
        )

    def build_top_applications_chart(self, df):
        e6_df = df[(df["E6"] != "") & ~df["E6"].isnull()]
        applications_total_df = list(e6_df["E6"])
        applications_top_data = {}
        for applications_list in applications_total_df:
            applications_array = applications_list.split(";")
            for application in applications_array:
                if (
                    application.startswith("android:Total Traffic")
                    or application.startswith("android:android")
                    or application.startswith("android:com.huawei")
                    or application.startswith("android:com.samsung")
                    or application.startswith("android:com.oneplus")
                ):
                    continue
                if application in applications_top_data:
                    applications_top_data[application] += 1
                else:
                    applications_top_data[application] = 1
        applications_top_data = dict(
            sorted(applications_top_data.items(), key=lambda x: x[1], reverse=True)
        )
        applications_top_data = {
            k: applications_top_data[k] for k in list(applications_top_data)[:10]
        }
        e6_total_df = {
            "Application": list(applications_top_data.keys()),
            "Total": list(applications_top_data.values()),
        }
        e6_month_chart = px.bar(
            pd.DataFrame(data=e6_total_df),
            x="Total",
            y="Application",
            labels={"Application": "Aplicação"},
        )
        e6_month_chart.update_layout(yaxis={"categoryorder": "total ascending"})
        return e6_month_chart

    def __get_total_nationality_data(self, d1_month_nationality_total):
        total_nationality_data = {}
        for month in d1_month_nationality_total:
            month_data = d1_month_nationality_total[month]
            month_nationalities = month_data["Nationality"]
            month_total_nationality = month_data["Total"]
            for index in range(len(month_nationalities)):
                if month_nationalities[index] in total_nationality_data:
                    total_nationality_data[
                        month_nationalities[index]
                    ] += month_total_nationality[index]
                else:
                    total_nationality_data[
                        month_nationalities[index]
                    ] = month_total_nationality[index]
        return total_nationality_data

    def __get_d1_month_series(
        self, total_nationality_data, d1_month_nationality_total, months_labels
    ):
        d1_month_series = []
        for nationality in total_nationality_data:
            nationality_data = []
            for month in months_labels:
                month_data = d1_month_nationality_total[month]
                month_nationalities = month_data["Nationality"]
                month_total_nationality = month_data["Total"]
                try:
                    index = month_nationalities.index(nationality)
                    nationality_total = month_total_nationality[index]
                except ValueError:
                    continue

                nationality_data.append(int(nationality_total))

            d1_month_series.append(
                {
                    "name": nationality,
                    "type": "bar",
                    "stack": "total",
                    "label": {"show": True},
                    "emphasis": {"focus": "series"},
                    "data": nationality_data,
                }
            )
        return d1_month_series

    def build_top_nationalities_chart(self, df, months_labels):
        d1_df = df[(df["D1"] != "") & (~df["D1"].isnull())]
        d1_month_nationality_total = {}
        for month in months_labels:
            month_df = d1_df[d1_df["Month"] == month]
            month_nacionalities_top_data = {}
            for row_index in month_df.index:
                nacionalities_list = month_df["D1"][row_index]
                nacionalities_array = nacionalities_list.split(";")
                if len(nacionalities_array) > 1:
                    continue
                nacionality = nacionalities_array[0]
                if nacionality in month_nacionalities_top_data:
                    month_nacionalities_top_data[nacionality] += month_df["C2"][
                        row_index
                    ]
                else:
                    month_nacionalities_top_data[nacionality] = month_df["C2"][
                        row_index
                    ]

            month_nacionalities_top_data = {
                k: month_nacionalities_top_data[k]
                for k in list(month_nacionalities_top_data)[:10]
            }
            d1_month_nationality_total[month] = {
                "Nationality": list(month_nacionalities_top_data.keys()),
                "Total": list(month_nacionalities_top_data.values()),
            }

        # Nationality total
        total_nationality_data = self.__get_total_nationality_data(
            d1_month_nationality_total
        )

        # Order total
        total_nationality_data = dict(
            sorted(total_nationality_data.items(), key=lambda x: x[1], reverse=True)
        )
        total_nationality_data = {
            k: total_nationality_data[k] for k in list(total_nationality_data)[:10]
        }
        d1_month_series = self.__get_d1_month_series(
            total_nationality_data, d1_month_nationality_total, months_labels
        )
        return {
            "tooltip": {"trigger": "axis", "axisPointer": {"type": "shadow"}},
            "legend": {"data": list(total_nationality_data.keys())},
            "grid": {
                "left": "3%",
                "right": "4%",
                "bottom": "3%",
                "containLabel": True,
            },
            "xAxis": {"type": "value"},
            "yAxis": {"type": "category", "data": list(months_labels)},
            "series": d1_month_series,
        }

    def build_total_people_evolution_chart(self, df, min_hour, max_hour):
        hours_str_array = []
        c1_evolution = []
        c2_evolution = []
        c1_c2_evolution = []
        minutes = [0, 15, 30, 45]
        for hour in range(min_hour, max_hour):
            for minute in minutes:
                hours_str_array.append(f"{hour:02d}:{minute:02d}")
                df_minutes = df[(df["Hour"] == hour) & (df["Minute"] == minute)]
                c1_evolution.append(f"{df_minutes['C1'].median():.2f}")
                c2_evolution.append(f"{df_minutes['C2'].median():.2f}")
                c1_c2_evolution.append(
                    f"{df_minutes['C1'].median() + df_minutes['C2'].median():.2f}"
                )

        return {
            "tooltip": {
                "trigger": "axis",
                "axisPointer": {
                    "type": "cross",
                    "label": {"backgroundColor": "#6a7985"},
                },
            },
            "legend": {"data": list(self.constants.c_columns.values())},
            "grid": {
                "left": "3%",
                "right": "4%",
                "bottom": "3%",
                "containLabel": True,
            },
            "xAxis": [
                {
                    "type": "category",
                    "boundaryGap": False,
                    "data": hours_str_array,
                }
            ],
            "yAxis": [{"type": "value"}],
            "series": [
                {
                    "name": self.constants.c_columns["C1"],
                    "type": "line",
                    "areaStyle": {},
                    "emphasis": {"focus": "series"},
                    "data": c1_evolution,
                },
                {
                    "name": self.constants.c_columns["C2"],
                    "type": "line",
                    "areaStyle": {},
                    "emphasis": {"focus": "series"},
                    "data": c2_evolution,
                },
                {
                    "name": self.constants.c_columns["C1 + C2"],
                    "type": "line",
                    "areaStyle": {},
                    "emphasis": {"focus": "series"},
                    "data": c1_c2_evolution,
                },
            ],
        }
